@@ -24,9 +24,9 @@ check() {
 }
 
 check master 
-check b2.24
+check b2.26
 check b2.23
-check $(logname) 
+check $(whoami) 
 
 echo "Status: $RET"
 ls -l $logdir/log.*
