@@ -1,12 +1,8 @@
 #!/bin/sh
-# Copyright (C) 2010 by Index Data ApS. 
-# http://www.indexdata.com
+# Copyright (C) 2010-2014 by Index Data ApS. http://www.indexdata.com
 #
 # build the engine with the lastest Index Data tools from GIT repo
 #
-# To install devel tools, see the list of tools on
-# http://twiki.indexdata.dk/cgi-bin/twiki/view/ID/CFRepoInstallation
-
 
 # fail on error
 set -e
@@ -36,53 +32,37 @@ make=make
 
 PATH=/bin:/usr/bin; export PATH
 
-# where you installed MacPorts
-: ${MACOS_PORTS_PREFIX=/opt/local}
-
-pc=${MACOS_PORTS_PREFIX}/bin/pkg-config
-
-# cross compiling flags for MacOS
+# OS specific compile flags
 case `uname` in
-    Darwin ) 	
-# xulrunner 1.9 compiles only on i386, not x86. 
-# everythings need to be compiled as i386 !!!
-            MOZ_VERSION=`$pc --modversion libxul-embedding`
-            if [[ $MOZ_VERSION = 2* ]]
-            then 
-	     	  xulrunner_arch=
-            else 
-	     	  xulrunner_arch="-arch i386"
-            fi
-	     	LDFLAGS="$xulrunner_arch -L$MACOS_PORTS_PREFIX/lib"
-	     	CFLAGS="$xulrunner_arch -g -O"
-	     	CXXFLAGS="$xulrunner_arch -g -O"
-	     	PATH="$MACOS_PORTS_PREFIX/bin:$PATH"
-		;;
-
-    FreeBSD ) 	make=gmake 
-		PATH=$PATH:/usr/local/bin
-		LDFLAGS="-L/usr/local/lib"; export LDFLAGS
-		;;
-    Linux ) 	
+    Linux )
 	PKG_CONFIG_PATH=/usr/cf-xulrunner/lib/pkgconfig; export PKG_CONFIG_PATH
 	if [ -e /etc/redhat-release ]; then
 	   centos=`cat /etc/redhat-release `
 	   PKG_CONFIG_PATH=/usr/cf-xulrunner/lib64/pkgconfig
 	fi
 	# outdated autoconf on centos5
-	case $centos in 
+	case $centos in
 	    *5.8* ) PATH=/usr/local/bin:$PATH ;;
 	esac
-	;;	
+	;;
 esac
 
 ###################################################
+git_clone () {
+    url="$1"
+    repo=$(basename $url)
 
+    if [ ! -d $repo ]; then
+        git clone -q $url
+    else
+        ( cd $repo && git pull )
+    fi
+}
 
-git clone -q $git_repo/private/cf
-git clone -q $git_repo/pub/yaz
-git clone -q $git_repo/pub/yazpp
-git clone -q $git_repo/pub/metaproxy 
+git_clone $git_repo/private/cf
+git_clone $git_repo/pub/yaz
+git_clone $git_repo/pub/yazpp
+git_clone $git_repo/pub/metaproxy
 
 dir=`pwd`
 if [ -n "$cf_branch" ]; then
@@ -90,7 +70,7 @@ if [ -n "$cf_branch" ]; then
    git checkout -b $cf_branch origin/$cf_branch
    if [ -n "$cf_git_id" ]; then
        git checkout $cf_git_id
-   fi 
+   fi
 fi
 
 cat <<EOF
@@ -108,7 +88,7 @@ for software in yaz yazpp metaproxy
 do
     echo "build >>> $software <<<"
     cd "$dir/$software"
-    $nice ./buildconf.sh 
+    $nice ./buildconf.sh
     LDFLAGS="$LDFLAGS" CFLAGS="$CFLAGS" CXXFLAGS="$CXXFLAGS" $nice ./configure --prefix=$prefix
     $nice $make -j$ncpu INKSCAPE=true all
 
@@ -128,8 +108,7 @@ cd "$dir/cf/engine/src"
 $nice $make -j$ncpu YAZ_CONFIG=../../../yaz/yaz-config MP_CONFIG=../../../metaproxy/metaproxy-config
 
 case $command in
-    test | check | fullcheck | fulltest ) 
+    test | check | fullcheck | fulltest )
 	cd "$dir/cf" && $make check ;;
     *) echo "You may run now: $ cd cf; $make check";;
 esac
-
