@@ -83,7 +83,7 @@ ContentFilter.prototype = {
       }
 
       for (var i=0; i<this.whitelist.length; i++) {
-        var re = new RegExp(this.whitelist[i]);
+        var re = this.whitelist[i];
         if (re.test(contentLocation.spec)) {
 //           dump ("WHITELISTED: " + contentLocation.spec + "\n");
           return ACCEPT;
@@ -91,7 +91,7 @@ ContentFilter.prototype = {
       }
 
       for (var i=0; i<this.blacklist.length; i++) {
-        var re = new RegExp(this.blacklist[i]);
+        var re = this.blacklist[i];
         if (re.test(contentLocation.spec)) {
           dump ("BLACKLISTED: " + contentLocation.spec + "\n");
           return REJECT_REQUEST;
@@ -112,7 +112,7 @@ ContentFilter.prototype = {
 
   // for nsIContentFilter
   addBlacklist: function (re) {
-   this.blacklist.push(re);
+   this.blacklist.push(new RegExp(re));
   },
 
   resetBlacklist: function () {
@@ -120,7 +120,7 @@ ContentFilter.prototype = {
   },
 
   addWhitelist: function (re)  {
-    this.whitelist.push(re);
+    this.whitelist.push(new RegExp(re));
   },
 
   resetWhitelist: function ()  {
