@@ -297,7 +297,7 @@ Block.prototype.handleError = function (e) {
   //follow the stack
   if (this.nextBlock) {
     logger.debug("Error handling passed to the next block " + this.debugname() );
-    this.nextBlock.handleError();
+    this.nextBlock.handleError(e);
     return;
   }
   logger.debug("Block ["+this.debugname()+"] will attempt to handle error");
