@@ -7,6 +7,7 @@ Components.utils.import('resource://indexdata/util/objHelper.js');
 Components.utils.import('resource://indexdata/util/helper.js');
 Components.utils.import("resource://indexdata/util/logging.js");
 var logger = logging.getLogger('');
+
 var contentFilter = Components.classes["@indexdata.com/contentfilter;1"]
                      .getService().wrappedJSObject;
 var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
@@ -59,7 +60,7 @@ Connector.prototype = {
     } else {
       // try to get the current tab
       var mainWindow = wm.getMostRecentWindow("navigator:browser");
-      if (mainWindow) pageWindow = mainWindow.gBrowser;
+      if (mainWindow) pageWindow = mainWindow.gBrowser.contentWindow;
     }
     if (pageWindow && (pageWindow.contentDocument || pageWindow.document))
       return pageWindow;
@@ -317,8 +318,12 @@ Connector.prototype = {
   loadConf: function (doc, session) {
     core.lastconnector = ""; // forget what we had before
     if (doc.getElementsByTagName("connector").length !== 1 ||
-        doc.firstChild.nodeName.toLowerCase() !== "connector")
+        doc.firstChild.nodeName.toLowerCase() !== "connector"){
+      logger.error("Looks like a bad connector. " +
+         " length=" + doc.getElementsByTagName("connector").length +
+         " node='" + doc.firstChild.nodeName + "'" );
       throw new Error("file is not a proper connector")
+    }
     var cn = doc.firstChild;
     if (cn.hasAttribute('template')) {
       this.template = new ConnectorTemplate();
