@@ -80,7 +80,10 @@ Step.prototype.toString = function () {
 }
 // Used arguments do not need to be unique as they are aggregated in the Task
 // TODO - Get from conf, to make it more automatic
-Step.prototype.getUsedArgs = function () { return []; };
+Step.prototype.getUsedArgs = function () {
+  return jsonPathHelper.usedArgs(this);
+};
+
 // reset run state
 Step.prototype.resetState = function () { }; 
 // return true if the step has been deprecated and should no longer be used
