@@ -6,7 +6,6 @@ Components.utils.import("resource://indexdata/util/Subject.js");
 **/
 var core = new Subject ([
   // nsILocalFile for the main CF directory ie. CF_BASE_PATH
-  'browser',
   'cfRootFile',
   'inBuilder',
   'platformVersion', // Xulrunner or Firefox
@@ -30,14 +29,3 @@ core.set('inBuilder', true);
 // Mostly used for debugging, so it may as well be a full file path
 // or repo URL.
 core.lastconnector = ""; // may be empty if not known
-
-core.getBrowser = function () {
-  if (this.browser) return this.browser;
-  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-           .getService(Components.interfaces.nsIWindowMediator);
-  var mainWindow = wm.getMostRecentWindow("navigator:browser")
-  return mainWindow.gBrowser.mCurrentBrowser;
-};
-core.setBrowser = function (browser) {
-  this.browser = browser;
-};
