@@ -18,7 +18,11 @@ Copy.prototype.draw = function(surface) {
 
 Copy.prototype.run = function (task) {
   jsonPathHelper.map(this.conf.in, this.conf.out, function (value) {
-    return value;
+    if (typeof value === 'object') {
+      // it's called Copy not Reference :)
+      return JSON.parse(JSON.stringify(value));
+    }
+    else return value;
   }, task.data);
 };
 
