@@ -8,7 +8,8 @@ CreateContainer.prototype = new Step();
 CreateContainer.prototype.constructor = CreateContainer;
 CreateContainer.prototype.init = function(task) { };
 CreateContainer.prototype.draw = function(surface) {
-  xulHelper.jsonPathField(surface, this, this.conf, 'target', "Create at: ");
+  xulHelper.jsonPathField(surface, this, this.conf, 'target', "Create at: ",
+    null, { rwmode:"w", containermode:"anynode" } );
 };
 CreateContainer.prototype.run = function (task, block) {
   jsonPathHelper.set(this.conf.target, {}, task.data);
