@@ -7,7 +7,10 @@ Delete.prototype = new Step();
 Delete.prototype.constructor = Delete;
 Delete.prototype.init = function() {};
 Delete.prototype.draw = function(surface) {
-  xulHelper.jsonPathField(surface, this, this.conf, "target", "");
+  xulHelper.jsonPathField(surface, this, this.conf, "target", "",
+                    null, { rwmode:"r", containermode:"anynode" } );
+    // marked as a read mode, since we are not writing into it, and
+    // do not want the replace/append options
 };
 Delete.prototype.run = function (task) {
   jsonPathHelper.set(this.conf.target, undefined, task.data)
