@@ -7,18 +7,24 @@ Components.utils.import('resource://indexdata/util/textHelper.js');
 Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/xulHelper.js');
 Components.utils.import('resource://indexdata/util/jsonPathHelper.js');
+
 var Download = function () { this.conf = {}; };
+
 Download.prototype = new Step();
 Download.prototype.constructor = Download;
 Download.prototype.init = function(task) {};
+
 Download.prototype.draw = function(surface) {
-  xmlHelper.appendNode(surface, "caption", "URL to retrieve:");
-  xulHelper.jsonPathField(surface, this, this.conf, "urljp", null,
-    {path:"$.temp", key:"url"});
+  xulHelper.jsonPathField(surface, this, this.conf, "urljp", "URL to retrieve:",
+    {path:"$.temp", key:"url"},
+    {rwmode:"r", singlevalue:true, constantallowed:true }  );
   xmlHelper.appendNode(surface, "caption", "Path and filename:");
   xulHelper.jsonPathField(surface, this, this.conf, "filenamejp", null,
-    {path:"$.output", key:"filename", append:jsonPathHelper.REPLACE});
+    {path:"$.temp", key:"filename", append:jsonPathHelper.REPLACE},
+    {rwmode:"w"} );
+    
 };
+
 Download.prototype.run = function (task) {
   var url = jsonPathHelper.getFirst(this.conf.urljp, task.data);
   if (!url) throw new StepError("Source URL not specified");
