@@ -22,9 +22,11 @@ ForEach.prototype.init = function(task) {
 
 ForEach.prototype.draw = function(surface) {
   let sourceBox = xulHelper.jsonPathField(surface, this, this.conf, 'source',
-              "Iterate through objects matching path (key ignored): ");
+              "Iterate through objects matching path (key ignored): ", null,
+              { containermode: "containerpath", rwmode: "r" } );
   let currentBox = xulHelper.jsonPathField(surface, this, this.conf, 'current', 
-              "Store current object: ", {path: '$.temp', key: 'current'});
+              "Store current object: ", {path: '$.temp', key: 'current'},
+              {rwmode:"w"} );
 };
 
 // Reset the run state, so we will start looping from the beginning
