@@ -6,6 +6,10 @@ Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/xulHelper.js');
 Components.utils.import('resource://indexdata/util/jsonPathHelper.js');
 Components.utils.import('resource://indexdata/runtime/Conditional.js');
+Components.utils.import('resource://indexdata/util/logging.js');
+
+var logger = logging.getLogger();
+
 var IfElse = function () {
  this.conf = {};
  this.selectedTab = 0;
@@ -93,7 +97,8 @@ IfElse.prototype.draw = function(surface) {
       style: 'min-width: 10px; margin: 0;' }); 
   right.addEventListener("click", function (e) {
     let sel = tabs.selectedIndex;
-    if ( typeof(sel) !== 'number' || sel === context.conf.conditionses.length - 2) return;
+    if ( typeof(sel) !== 'number' || sel === context.conf.conditionses.length - 2)
+      return;
     let removed = context.conf.conditionses.splice(sel, 1)[0]; 
     context.conf.conditionses.splice(sel+1, 0, removed); 
     let removed = context.blocks.splice(sel, 1)[0]; 
@@ -105,12 +110,14 @@ IfElse.prototype.draw = function(surface) {
       style: 'min-width: 10px; margin: 0;' }); 
   remove.addEventListener("click", function (e) {
     let sel = tabs.selectedIndex;
-    if ( typeof(sel) !== 'number' || context.conf.conditionses.length < 2) return;
+    if ( typeof(sel) !== 'number' || context.conf.conditionses.length < 2)
+      return;
     context.conf.conditionses.splice(sel, 1)[0]; 
     context.blocks.splice(sel, 1)[0]; 
     refresh(sel > 1 ? sel-1 : 0);
   }, false);
-};
+}; // draw
+
 IfElse.prototype.run = function (task, block) {
   for (let i = 0; i < this.conditionals.length; i++) {
     if (this.conditionals[i].eval()) {
@@ -119,21 +126,64 @@ IfElse.prototype.run = function (task, block) {
     }
   }
 };
-IfElse.prototype.getClassName = function () { return "IfElse"; };
-IfElse.prototype.getDisplayName = function () { return "Conditional"; };
-IfElse.prototype.getDescription = function () { return "If... Else If... Else"; };
-IfElse.prototype.getVersion = function () { return "1.2"; };
+
+IfElse.prototype.getClassName = function () {
+  return "IfElse";
+};
+
+IfElse.prototype.getDisplayName = function () {
+  return "Conditional";
+};
+
+IfElse.prototype.getDescription = function () {
+  return "If... Else If... Else";
+};
+
+IfElse.prototype.getVersion = function () {
+  //return "1.2";
+  return "2.0";  // using the new Conditional with constants built into jp
+};
+
+IfElse.prototype.upgrade = function (confVer, curVer, conf) {
+  // can't upgrade if the connector is newer than the step
+  logger.debug("if-else upgrade from " + confVer + " to " + curVer );
+  if (confVer > curVer)
+    return false;
+  if (confVer < 2.0) {
+    for (let i = 0; i < this.conf.conditionses.length ; i++) {
+      let c = new Conditional(this.conf.conditionses[i], null );
+      c.upgradeConstants();
+      this.conf.conditionses[i] = c.conditions;
+    }
+    logger.debug("if-else: upgraded conditions to " + JSON.stringify( conf.conditionses ) );
+  }
+  return true;
+};
+
+
 IfElse.prototype.capabilityFlagDefault = function (flag) {
   for (let i = 0; i < this.conf.conditionses.length; i++) {
     if (jsonPathHelper.conditionsCapable(this.conf.conditionses[i],
         flag, "$.input", "index")) { return true; }
   }
-}
-IfElse.prototype.getUsedArgs = function () { return []; }
+};
+
+IfElse.prototype.getUsedArgs = function () {
+  var u = [];
+  for ( let i = 0; i < this.conditionals.length; i++ ) {
+    u = u.concat ( this.conditionals[i].getUsedArgs() );
+  }
+  logger.debug("if-else: usedargs: " + JSON.stringify(u) );
+  return u;
+};
+
 IfElse.prototype.renderBlockArgs = function (i) {
-  if (this.blocks[i].label === 'else' ) return "";
+  if (this.blocks[i].label === 'else' )
+    return "";
   return String(this.conditionals[i]);
 };
+
 IfElse.prototype.renderArgs = function () {
-  if (Array.isArray(this.conditionals)) return "(" +this.conditionals.length + " cases)";
+  if (Array.isArray(this.conditionals))
+    return "(" +this.conditionals.length + " cases)";
 };
