@@ -7,7 +7,6 @@ Components.utils.import("resource://indexdata/util/logging.js");
 Components.utils.import('resource://indexdata/util/jsonPathHelper.js');
 Components.utils.import('resource://indexdata/util/logging.js');
 Components.utils.import('resource://indexdata/runtime/Task.js');
-Components.utils.import('resource://indexdata/ui/app.js');
 
 var logger = logging.getLogger();
 
@@ -27,6 +26,7 @@ Map.prototype.init = function() {};
 
 Map.prototype.draw = function(surface) {
   Components.utils.import('resource://indexdata/ui/taskPane.js');
+  Components.utils.import('resource://indexdata/ui/app.js');
   var context = this;
 
   // Data
@@ -212,7 +212,7 @@ Map.prototype.draw = function(surface) {
 
     var subdbs = app.connector.metaData["subdb"].split(/,\s*/);
     xmlHelper.emptyChildren(mapBox);
-    for (i=0; i<subdbs.length; i++) {
+    for (let i=0; i<subdbs.length; i++) {
       addMap( { in: subdbs[i], out: ""} );
     }
     saveMaps();
