@@ -50,6 +50,10 @@ NormalizeDate.regexps = [
         [ /^\s*(\d{4}) ([A-z]{3})\b/, 1, 2, null ],     // 1968 Mar
         [ /^\s*(\d{1,2})[ -]([A-z]{3})[ -](\d{4})\b/, 3, 2, 1 ], // 12-MAR-1968
         [ /^\s*([A-z]+) (\d+)((st)|(nd)|(rd)|(th))[, ]+(\d{4})\b/, 8, 1, 2 ], // Mar 12th 1968
+        [ /^\s*(\d{4})-(\d{2})-(\d{2})/, 1,2,3 ],  // ISO date 2014-06-18
+        [ /^\s*(\d{4})-(\d{2})/, 1,2,null ],  // ISO date 2014-06
+        //[ /^\s*(\d{4})\.(\d{2})\.(\d{2})/, 1,2,3 ],  // 2014.06.18 as on old Indexdata frontpage
+                                                     // and our regression test pages
         // And finally, the emergency backup regexp ...
         //      [ /(\d{4})/,                   1, null, null ],
 ];
@@ -105,17 +109,26 @@ NormalizeDate.normalize = function(s) {
       s = news;
   }
 
-  if (!s.match(/^\d{4}$/)) {
+  if (!s.match(/^\d{4}$/)  &&
+       !s.match(/^\d{4}-\d{2}/)  ) {
     // Special case: Date.parse() handles four-digit numbers, but
     // returns not the year but the first day of that year. Avoid
     // xinvoking it for such dates.
+    // Another special case: If it looks like an ISO date, starts with
+    // YYYY-MM  (day is optional here), we do not use date.parse, because
+    // it assumes UTC for ISO dates, and localtime for anything else. Since
+    // it defaults to 00:00, time zones can push it to another date.
+    // CP-3813
     var ms = Date.parse(s);
     if (ms) {
-      NormalizeDate.logger.debug("Date.parse() parsed '" + s + "'");
       var date = new Date(ms);
-      var y = date.getUTCFullYear();
-      var m = date.getUTCMonth()+1; // *sigh*
-      var d = date.getUTCDate();
+      NormalizeDate.logger.debug("Date.parse() parsed '" + s + "' to " + ms + " which is " + date);
+      //var y = date.getUTCFullYear();
+      //var m = date.getUTCMonth()+1; // *sigh*
+      //var d = date.getUTCDate();
+      var y = date.getFullYear();
+      var m = date.getMonth()+1; // *sigh*
+      var d = date.getDate();
       return y + "-" + ("00"+m).substr(-2) + "-" + ("00"+d).substr(-2);
     }
   }
@@ -254,8 +267,20 @@ NormalizeDate.prototype.unitTest = function () {
 	[ "1968 MAR",					"1968-03" ],
 	[ "12-MAR-1968",				"1968-03-12" ],
 	[ "Mar 12th 1968",				"1968-03-12" ],
+        [ "2014 06 18",                                 "2014-06-18" ],
+
+        // ISO dates, parsed with a regexp. CP-3813
+        [ "2014-06-18",                                 "2014-06-18" ],
+        [ "1968-03-12",                                 "1968-03-12" ],
+        [ "1968-03",                                 "1968-03" ],
+        [ "1968-03-12T00:00:00+0200",                                 "1968-03-12" ],
+
+        // This fails, returns 1968-03-11, because the timezone and midnight
+        // at least when run in Copenhagen
+        //[ "Mar 12, 1968 00:00:00 GMT+1200",           "1968-03-12" ],
     ];
 
+    var ok = true;
     for (var i = 0; i < samples.length; i++) {
 	var ref = samples[i];
 	var xin = ref[0];
@@ -265,8 +290,9 @@ NormalizeDate.prototype.unitTest = function () {
 	    logger.info("parsed '" + xin + "' correctly");
 	} else {
 	    logger.error("parsed '" + xin + "' to '" + out + "' (expected '" + xout + "')");
+            ok = false;
 	}
     }
 
-    return true;
+    return ok;
 };
