@@ -34,13 +34,7 @@ ScriptSandbox.prototype.draw = function(surface) {
 };
 ScriptSandbox.prototype.run = function (task) {
   var step = this;
-  // in the engine the getPageWindow returns an IFRAME
-  // so retrieve from the doc instead
-  if (core.inBuilder) {
-    var pageWin = task.connector.getPageWindow().contentWindow;
-  } else {
-    var pageWin = this.getPageDoc().defaultView;
-  }
+  var pageWin = task.connector.getPageWindow();
   var sb = new Components.utils.Sandbox(pageWin);
   sb.window = new XPCNativeWrapper(pageWin);
   sb.document = sb.window.document; 
