@@ -4,14 +4,16 @@ Components.utils.import('resource://indexdata/runtime/StepError.js');
 Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/xulHelper.js');
 Components.utils.import('resource://indexdata/util/jsonPathHelper.js');
+Components.utils.import('resource://indexdata/util/logging.js');
+
+var logger = logging.getLogger();
 
 var SetValue = function () {
   // this is only used as the init values for the display if no configuration
   // is provided, e.g when new step is added to the task
   this.conf = {
       dest: '',
-      param: undefined,
-      constant: undefined,
+      param: { path:"", key:"" },
       usejp: false,
       // One of 'empty', 'fail', or 'noop'
       onempty: 'empty'
@@ -30,12 +32,6 @@ SetValue.prototype.draw = function(surface) {
   // if you're setting a select element you can pick an option to populate constant
   // should you decide not to we stop listening once you move into the step pane
   var selectNode = null;
-  var grabOption = function(e) {
-    context.conf['constant'] = selectNode.value;
-    constInput.value = selectNode.value;
-    radiogroup.selectedItem = constRadio;
-    selectNode.removeEventListener("change", grabOption, false);
-  };
   var onNodeSelect = function(node) {
     if (node.localName.toLowerCase() == "select") {
       selectNode = node;
@@ -43,18 +39,17 @@ SetValue.prototype.draw = function(surface) {
     }
   };
 
-  var hbox = xmlHelper.appendNode(radiogroup, "hbox", null, {align:"center"}, null);
+  var hbox = xmlHelper.appendNode(surface, "hbox", null, {align:"center"}, null);
   xmlHelper.appendNode(hbox, "label", "Form field to populate: ");
   var nodePicker = xulHelper.singleNodeField(hbox, this, "dest", onNodeSelect);
 
-  hbox = xmlHelper.appendNode(radiogroup, "hbox", null, {align:"center"}, null);
-  var argRadio = xmlHelper.appendNode(hbox, "radio", null, {"id":"argRadio", "label":"Populate with stored data:"});
+  hbox = xmlHelper.appendNode(surface, "hbox", null, {align:"center"}, null);
 
-  var jpField = xulHelper.jsonPathField(hbox, this, this.conf, "param");
+  var jpField = xulHelper.jsonPathField(hbox, this, this.conf, "param", "Value", null,
+        { rwmode:"r", singlevalue: true,
+          constantallowed: true,  expandvariables: true,
+        } );
 
-  hbox = xmlHelper.appendNode(radiogroup, "hbox", null, {align:"center"}, null);
-  var constRadio = xmlHelper.appendNode(hbox, "radio", null, {"id":"constRadio", "label":"Populate with constant:"});
-  var constInput = xmlHelper.appendNode(hbox, "textbox", null, {"value":this.conf["constant"]||"", "flex":"1"});
 
   xmlHelper.appendNode(surface, "separator", null, {'class': 'groove'});
   xmlHelper.appendNode(surface, "caption", null, {'label':'When source empty:'});
@@ -66,41 +61,12 @@ SetValue.prototype.draw = function(surface) {
   emptyRadioGroup.selectedItem = emptyRadios[this.conf.onempty];
 
 
-  // reflect saved values
-  if (context.conf.usejp) radiogroup.selectedItem = argRadio;
-  else radiogroup.selectedItem = constRadio;
-
-  var selectJP = function (e) {
-    context.task.debug("Selected JSONpath.");
-    context.conf.usejp = true;
-    context.draw(surface);
-  };
-
-  var selectConstant = function (e) {
-    context.task.debug("Selected constant.");
-    context.conf.usejp = false;
-    context.conf["constant"] = constInput.value;
-  };
 
   emptyRadioGroup.addEventListener('select', function(e) {
     context.conf.onempty = e.target.value;
   }, false);
 
-  radiogroup.addEventListener("select", function(e) {
-    if (e.target.localName === "radiogroup") {
-      if (!context.conf.usejp && radiogroup.selectedItem === argRadio) selectJP(e);
-      else if (radiogroup.selectedItem === constRadio) selectConstant(e);
-    }
-  }, false);
 
-  jpField.addEventListener("command", function (e) { radiogroup.selectedItem = argRadio; }, false);
-  jpField.addEventListener("input", function (e) { radiogroup.selectedItem = argRadio; }, false);
-  constInput.addEventListener("input", function (e) { radiogroup.selectedItem = constRadio; }, false);
-
-  // if user opted not to take an option from the select node, unbind the listener
-  surface.addEventListener("mouseover", function (e) {
-    if (selectNode) selectNode.removeEventListener("change", grabOption, false);
-  }, false);
 };
 
 
@@ -109,20 +75,15 @@ SetValue.prototype.run = function (task) {
   let nodeSpec = jsonPathHelper.inlineReplaceNodeSpec(this.conf.dest, task.data);
   let dest = xmlHelper.getElementByNodeSpec(this.getPageDoc(), nodeSpec);
   if(!dest) throw new StepError("destination element not found");
-  if (this.conf.usejp) {
-    var val = jsonPathHelper.getFirst(this.conf.param, task.data);
-    var container = this.conf['param'].key;
-  } else {
-    var val = this.conf.constant;
-    var container = 'constant';
-  }
+  var val = jsonPathHelper.getFirst(this.conf.param, task.data);
   // 0 is the only falsy value that doesn't count as empty
   if (val || typeof val === 'number') {
     dest.value = val; 
-    task.info('value set to "' + val + '" from ' + container, this);
+    task.info('value set to "' + val + '" from ' +
+        jsonPathHelper.displayString(this.conf.param,true), this );
   } else if (this.conf.onempty === 'empty') {
     dest.value = "";
-    task.info('value set to an empty string, ' + container + ' is ' + val, this);
+    task.info('value set to an empty string, from "'  + val + '"', this);
   } else if (this.conf.onempty === 'fail') {
     throw new StepError("empty value");
   } else if (this.conf.onempty === 'noop') {
@@ -133,12 +94,11 @@ SetValue.prototype.run = function (task) {
   dest.dispatchEvent(changeEvent);
 };
 
+/* The (new) default in Step.js seems to be good enough 
 SetValue.prototype.getUsedArgs = function () {
- if (this.conf.param && this.conf.param.path === "$.input")
-   return [this.conf.param.key];
- else
-   return [];
-}
+  return jsonPathHelper.usedArgs(this, "param");
+};
+*/
 
 SetValue.prototype.getClassName = function () {
   return "SetValue";
@@ -153,7 +113,8 @@ SetValue.prototype.getDescription = function () {
 };
 
 SetValue.prototype.getVersion = function () {
-  return "4.0";
+     // 4.0 introduced onempty 
+  return "4.1";  // 4.1 moved the constant into the jp
 };
 
 SetValue.prototype.upgrade = function (confVer, curVer, conf) {
@@ -172,20 +133,42 @@ SetValue.prototype.upgrade = function (confVer, curVer, conf) {
       conf.param = {path: "$.input", key: conf.param};
   }
   if (confVer < 2.0) {
-    if (conf.constant) conf.usejp = false;
-    else conf.usejp = true;
+    if (conf.constant)
+      conf.usejp = false;
+    else
+      conf.usejp = true;
   }
   if (confVer < 4.0) {
-    if (!conf.usejp) conf.onempty = 'fail';
-    else conf.onempty = 'noop';
+    if (!conf.usejp)
+      conf.onempty = 'fail';
+    else
+      conf.onempty = 'noop';
+  }
+  if (confVer < 4.1) {
+    logger.debug("SetValue upgrade from " + confVer + " conf: " + JSON.stringify(conf) );
+    if ( ! conf.param )
+      conf.param = { path:"", key:"" };
+    conf.param.useconstant =  ! conf.usejp;
+    conf.param.constantvalue = conf.constant;
+    if ( typeof(conf.param.constantvalue) == "string" &&
+         conf.param.constantvalue.match( /\{\$./ ) ) {
+      conf.param.expandvariables = true;
+    }
+    conf.constant = undefined;
+    conf.usejp = undefined;
+    logger.debug("SetValue upgrade done " + JSON.stringify(conf) );
+
   }
   return true;
 };
 
 SetValue.prototype.renderArgs = function () {
-  if (!this.conf.dest) return "";
-  else if (this.conf.usejp) return this.conf.param.path + "." + this.conf.param.key;
-  else return this.conf.constant;
+  if (!this.conf.dest)
+    return "";
+  else if (this.conf.usejp)
+    return this.conf.param.path + "." + this.conf.param.key;
+  else
+    return this.conf.constant;
 }
 
 
