@@ -36,6 +36,8 @@ function getToSkipType(me, stepsToSkip) {
 }
 
 Skip.prototype.draw = function(surface) {
+  logger.debug("skip.draw: conf: " + JSON.stringify(this.conf) );
+  logger.debug("skip.draw: cond: " + JSON.stringify(this.conditional) );
   xmlHelper.appendNode(surface, "caption", "Only run next step(s) if:");
   var condBox = xmlHelper.appendNode(surface, "vbox");
   this.conditional.draw(condBox, this);
@@ -131,7 +133,8 @@ Skip.prototype.getDescription = function () {
 };
 
 Skip.prototype.getVersion = function () {
-  return "2.0";
+  //return "2.0";
+  return "3.0";  // uses the improved conditional, with constants inside jp
 };
 
 Skip.prototype.upgrade = function (confVer, curVer, conf) {
@@ -159,6 +162,12 @@ Skip.prototype.upgrade = function (confVer, curVer, conf) {
     delete conf.rel;
     delete conf.useRegex;
   }
+  if (confVer < 3.0) {
+    var c = new Conditional(conf.conditions, null);
+    c.upgradeConstants();
+    conf.conditions = c.conditions;
+    logger.debug("skip: upgraded conditions to " + JSON.stringify( conf.conditions) );
+  }
   return true;
 };
 
