@@ -48,8 +48,30 @@ While.prototype.getDisplayName = function () { return "While"; };
 While.prototype.getDescription = function () {
   return "Repeat block while a condition is true.";
 };
-While.prototype.getVersion = function () { return "1.0"; };
-While.prototype.getUsedArgs = function () { return []; }
+While.prototype.getVersion = function () {
+  //return "1.0";
+  return "2.0"; // using the new Conditional with constants inside the jp
+};
+While.prototype.upgrade = function (confVer, curVer, conf) {
+  // can't upgrade if the connector is newer than the step
+  if (confVer > curVer)
+    return false;
+  if (confVer < 2.0) {
+    var c = new Conditional(conf.conditions, null);
+    c.upgradeConstants();
+    conf.conditions = c.conditions;
+    logger.debug("skip: upgraded conditions to " + JSON.stringify( conf.conditions) );
+  }
+  return true;
+};
+
+While.prototype.getUsedArgs = function () {
+ if (this.conditional)
+  return this.conditional.getUsedArgs();
+ return [];
+}
+
 While.prototype.renderArgs = function () { 
- if (this.conditional) return String(this.conditional);
+ if (this.conditional)
+   return String(this.conditional);
 };
