@@ -1,17 +1,12 @@
 var EXPORTED_SYMBOLS = ["repoClient","uriListener"];
 Components.utils.import("resource://indexdata/util/xmlHelper.js");
+Components.utils.import("resource://indexdata/runtime/core.js");
 Components.utils.import("resource://indexdata/ui/app.js");
 Components.utils.import("resource://indexdata/util/logging.js");
 Components.utils.import("resource://indexdata/util/io.js");
-
 const NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-
 var logger = logging.getLogger();
-
-//builder pref, id branch
-var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-  .getService(Components.interfaces.nsIPrefService)
-  .getBranch("indexdata.cf.");
+var idprefs = core.idprefs;
 
 /*
  * Provides entry point for handling files with application/cf+xml type
@@ -101,6 +96,7 @@ cfContentHandler.prototype = {
         logger.debug(e.message);
         app.connector.metaData.origin = "";
       }
+      core.lastconnector = this.origin;
       app.connector.useSessionTestValues();
       app.notify("connector");
       app.set("lastOpened", null);
@@ -160,7 +156,7 @@ var repoClient = {
   },
 
   homerepo_url: function () {
-    var url = prefs.getCharPref("homerepo");
+    var url = idprefs.getCharPref("homerepo");
     url = url.replace(/\/+$/, ""); // trim trailing slashes
     logger.debug("homerepo url: " + url);
     return url;
@@ -247,7 +243,7 @@ var repoClient = {
     //built ninja request url
     var ninjaURL = null;
     try {
-      ninjaURL = prefs.getCharPref("ninjaurl");
+      ninjaURL = idprefs.getCharPref("ninjaurl");
     } catch (e) {
       logger.debug(e);
     }
@@ -258,7 +254,7 @@ var repoClient = {
     }
     var ninjaSearch = null;
     try {
-      ninjaSearch = prefs.getCharPref("ninja.searchengine");
+      ninjaSearch = idprefs.getCharPref("ninja.searchengine");
     } catch (e) {
       logger.debug(e);
     }
@@ -362,7 +358,7 @@ var repoClient = {
     //validate meta-data
     var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
       .getService(Components.interfaces.nsIPromptService);
-    var metaDefs = app.template.getMetaData();
+    var metaDefs = app.connector.template.getMetaData();
     for (var i=0; i<metaDefs.length; i++) {
       var validate = metaDefs[i].validate;
       if (validate && validate === "non-empty") {
@@ -427,7 +423,7 @@ var repoClient = {
     //ask for commit message
     var user = null;
     try {
-      user = prefs.getCharPref("username");
+      user = idprefs.getCharPref("username");
     } catch (e) {
       logger.debug("Pref 'username' not found, will try current user");
     }
@@ -454,7 +450,7 @@ var repoClient = {
     }
     var comMsg = check.value ? user +": "+ input.value : input.value;
     app.connector.metaData.lastCommit = comMsg;
-    if (app.version) app.connector.metaData.cfbuilder_version = app.addon.version;
+    if (core.version) app.connector.metaData.cfbuilder_version = core.version;
 
     var originURL = this.homerepo_url() + "/" + origin;
     logger.debug("Pushing to " + originURL);
