@@ -7,7 +7,7 @@ var Highlighter = function (domDoc, cb) {
   this.doc = domDoc;
   this.cb = cb;
   if ( domDoc.cf_displaydoc ) {
-    //dump("Highlighter, using display doc \n");
+    logger.debug("Highlighter, using display doc \n");
     this.doc = domDoc.cf_displaydoc;
   }
   this.outlineStyle = '2px dashed #ff7deb';
