@@ -47,12 +47,14 @@ var justObserve = {
     OBSRV.addObserver(this, "http-on-modify-request", false);
     OBSRV.addObserver(this, "http-on-examine-response", false);
     OBSRV.addObserver(this, "http-on-examine-cached-response", false);
+    OBSRV.addObserver(this, "http-on-examine-merged-response", false);
   },
   
   detatch: function () {
-      OBSRV.removeObserver(this, "http-on-modify-request");
-      OBSRV.removeObserver(this, "http-on-examine-response");
-      OBSRV.removeObserver(this, "http-on-examine-cached-response");  
+    OBSRV.removeObserver(this, "http-on-modify-request");
+    OBSRV.removeObserver(this, "http-on-examine-response");
+    OBSRV.removeObserver(this, "http-on-examine-cached-response");  
+    OBSRV.removeObserver(this, "http-on-examine-merged-response");
   },
   
   onTimeout: progressListen.onTimeout,
@@ -73,9 +75,13 @@ var justObserve = {
   observe: function(subject, topic, data) {
     subject.QueryInterface(Components.interfaces.nsIRequest);
     subject.QueryInterface(Components.interfaces.nsIHttpChannel);    
-    if (topic=="http-on-modify-request")
+    if (topic === "http-on-modify-request") {
       this.requestStart(subject, 'observer');
-    if (topic=="http-on-examine-response" || topic=="http-on-examine-cached-response")
+    }
+    if (topic === "http-on-examine-response"
+        || topic === "http-on-examine-cached-response"
+        || topic === "http-on-examine-merged-response")  {
       this.requestStop(subject, 'observer');
+    } 
   },
 };
