@@ -1,25 +1,27 @@
 var EXPORTED_SYMBOLS = ["progressListen"];
-Components.utils.import("resource://indexdata/runtime/core.js");
+var Cc = Components.classes;
+var Ci = Components.interfaces;
+const STATE_START = Ci.nsIWebProgressListener.STATE_START;
+const STATE_TRANSFERRING = Ci.nsIWebProgressListener.STATE_TRANSFERRING;
+const STATE_STOP = Ci.nsIWebProgressListener.STATE_STOP;
+const STATE_REDIRECTING = Ci.nsIWebProgressListener.STATE_REDIRECTING;
+const STATE_IS_DOCUMENT = Ci.nsIWebProgressListener.STATE_IS_DOCUMENT;
+const NOTIFY_ALL = Ci.nsIWebProgress.NOTIFY_ALL;
 
-const STATE_START = Components.interfaces.nsIWebProgressListener.STATE_START;
-const STATE_TRANSFERRING = Components.interfaces.nsIWebProgressListener.STATE_TRANSFERRING;
-const STATE_STOP = Components.interfaces.nsIWebProgressListener.STATE_STOP;
-const STATE_REDIRECTING = Components.interfaces.nsIWebProgressListener.STATE_REDIRECTING;
-const STATE_IS_DOCUMENT = Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT;
-const NOTIFY_ALL = Components.interfaces.nsIWebProgress.NOTIFY_ALL;
-
-const OBSRV = Components.classes["@mozilla.org/observer-service;1"]
-  .getService(Components.interfaces.nsIObserverService);
+const OBSRV = Cc["@mozilla.org/observer-service;1"]
+  .getService(Ci.nsIObserverService);
 
 var progressListen = {
   started: null,
-  browser: null,
+  progress: null,
   task: null,
+  isEngine: false,
   redirectedFrom: null,
   waitXHR: true,
+  minQuiet: 50,
   quietTime: 50,
-  quietTimer: Components.classes["@mozilla.org/timer;1"]
-    .createInstance(Components.interfaces.nsITimer),
+  quietTimer: Cc["@mozilla.org/timer;1"]
+    .createInstance(Ci.nsITimer),
   defaultConf: {
     quietTime: 50,
     waitXHR: true,
@@ -50,7 +52,12 @@ var progressListen = {
   },
 
   wait: function(task, conf, step) {
-    this.browser = core.getBrowser();
+    this.progress = task.connector.getPageWindow();
+    if (!this.progress.addProgressListener)  {
+      this.progress = Cc["@mozilla.org/docloaderservice;1"].getService(Ci.nsIWebProgress);
+      this.isEngine = true;
+    }
+    this.progress.addProgressListener(progressListen, NOTIFY_ALL);
     this.task = task;
     this.step = step;
 
@@ -63,25 +70,28 @@ var progressListen = {
       this.quietTime = conf.quietTime;
     else
       this.quietTime = this.defaultConf.quietTime;
+    if (this.quietTime < this.minQuiet) this.quietTime = this.minQuiet;
 
-    this.started = {observer:[], progress:[]};
+    this.started = {observer:[], progress:[], progress_document:[]};
 
     if (this.waitXHR) {
       OBSRV.addObserver(this, "http-on-modify-request", false);
       OBSRV.addObserver(this, "http-on-examine-response", false);
       OBSRV.addObserver(this, "http-on-examine-cached-response", false);
+      OBSRV.addObserver(this, "http-on-examine-merged-response", false);
     }
-    this.browser.addProgressListener(this);
   },
 
   detatch: function () {
-      if (this.browser) this.browser.removeProgressListener(this);
-      this.browser = null;
-      if (this.waitXHR) {
-        OBSRV.removeObserver(this, "http-on-modify-request");
-        OBSRV.removeObserver(this, "http-on-examine-response");
-        OBSRV.removeObserver(this, "http-on-examine-cached-response");
-      }
+    if (this.progress)
+      this.progress.removeProgressListener(this);
+    this.progress = null;
+    if (this.waitXHR) {
+      OBSRV.removeObserver(this, "http-on-modify-request");
+      OBSRV.removeObserver(this, "http-on-examine-response");
+      OBSRV.removeObserver(this, "http-on-examine-cached-response");
+      OBSRV.removeObserver(this, "http-on-examine-merged-response");
+    }
   },
 
   onTimeout: function () {
@@ -100,10 +110,10 @@ var progressListen = {
   // this object handles.  Naturally we're a progress listener but we also
   // can provide weak references to ourself thanks to XPConnect.
   QueryInterface: function(aIID) {
-    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
-        aIID.equals(Components.interfaces.nsIObserver) ||
-        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
-        aIID.equals(Components.interfaces.nsISupports))
+    if (aIID.equals(Ci.nsIWebProgressListener) ||
+        aIID.equals(Ci.nsIObserver) ||
+        aIID.equals(Ci.nsISupportsWeakReference) ||
+        aIID.equals(Ci.nsISupports))
       return this;
     throw Components.results.NS_NOINTERFACE;
   },
@@ -113,7 +123,7 @@ var progressListen = {
     for (var i=0; i<started.length; i++) {
       if (started[i] === aRequest) {
         started.splice(i, 1);
-        this.task.debug("  REMOVED request: " + aRequest.name, this.step);
+        this.task.debug("REMOVED " + key + " request: " + aRequest.name, this.step);
         break;
       }
     }
@@ -141,7 +151,7 @@ var progressListen = {
     // Not all requests are HTTP channels
     try {
       //TODO: This works but why slow it down more, commenting just in case
-      //httpChannel = aRequest.QueryInterface(Components.interfaces.nsIHttpChannel);
+      //httpChannel = aRequest.QueryInterface(Ci.nsIHttpChannel);
     } catch (e) {
     }
     // TODO: This slows it down too much or something and responses get missed
@@ -173,7 +183,9 @@ var progressListen = {
     this.task.addRunDatumUrlEvent(aRequest.name, "stop", this.step);
     // Remove it from the array of pages we're waiting for
     this.removeRequest(aRequest, key);
-    if (this.keysEmpty(this.started)) {
+    if (this.keysEmpty(this.started.progress)
+         && this.keysEmpty(this.started.progress_document)
+         && this.keysEmpty(this.started.observer)) {
       var context = this;
       this.task.info("  All pages loaded, waiting.", this.step);
       // The array of pending pages is empty, wait to allow new requests to start.
@@ -183,32 +195,33 @@ var progressListen = {
           Components.utils.import('resource://indexdata/util/waitForLoad.js');
           waitForLoad.detatch();
         }
-      }, this.quietTime, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+      }, this.quietTime, Ci.nsITimer.TYPE_ONE_SHOT);
     }
   },
 
   observe: function(subject, topic, data) {
-    subject.QueryInterface(Components.interfaces.nsIRequest);
-    subject.QueryInterface(Components.interfaces.nsIHttpChannel);
-    if ((subject.notificationCallbacks)) {
-
-      try
-      {
-          var xhrRequest = subject.notificationCallbacks.getInterface(Components.interfaces.nsIXMLHttpRequest);
-          if (xhrRequest) {
- this.task.debug (topic +  " XHR OBSERVED! " + subject.name);
-            if (topic=="http-on-modify-request")
-              this.requestStart(subject, 'observer');
-            if (topic=="http-on-examine-response" || topic=="http-on-examine-cached-response")
-              this.requestStop(subject, 'observer');
-          }
+    let channel = subject.QueryInterface(Ci.nsIChannel);
+    if (topic === "http-on-examine-response"
+        || topic === "http-on-examine-cached-response"
+        || topic === "http-on-examine-merged-response")  {
+      this.requestStop(subject, 'observer');
+    } else if (topic === "http-on-modify-request") {
+      try {
+        let callbacks = channel.notificationCallbacks;
+        let xhrRequest = callbacks ?
+                           callbacks.getInterface(Ci.nsIXMLHttpRequest)
+                           : null;
+        // Only use the observer for XHR requests. To use the observer for
+        // all requests use justObserve.js
+        if (xhrRequest && topic === "http-on-modify-request") {
+          this.requestStart(subject, 'observer');
+        }
+      } catch (e) {
       }
-      catch (e) {}
-    }
+    }  
   },
 
   onStateChange: function(aWebProgress, aRequest, aFlag, aStatus) {
-    let started = this.started.progress;
     // This fires when the load event is initiated
 
     // Old dump code to get hex output
@@ -220,51 +233,35 @@ var progressListen = {
       var flagStrings = "STATE_START STATE_REDIRECTING STATE_TRANSFERRING STATE_NEGOTIATING STATE_STOP STATE_IS_REQUEST STATE_IS_DOCUMENT STATE_IS_NETWORK STATE_IS_WINDOW STATE_RESTORING STATE_IS_INSECURE STATE_IS_BROKEN STATE_IS_SECURE STATE_SECURE_HIGH STATE_SECURE_MED STATE_SECURE_LOW".split(" ");
       output = ("onStateChange FLAGS! " + req.name + " ::: ");
       for (var i=0; i<flagStrings.length; i++) {
-        if (eval("Components.interfaces.nsIWebProgressListener." + flagStrings[i] + " & flags"))
+        if (eval("Ci.nsIWebProgressListener." + flagStrings[i] + " & flags"))
           output += (flagStrings[i] + " ");
       }
       return output;
     };
 
     this.task.debug (wplFlagDump(aFlag, aRequest), this.step);
-    this.task.debug (started.length + " requests BEFORE:", this.step);
-    for (var i=0; i<started.length; i++, this.step) {
-      this.task.debug(i + ". " + started[i].name, this.step);
+    if (aFlag & STATE_IS_DOCUMENT) {
+      if (aFlag & STATE_START) {
+        this.requestStart(aRequest, 'progress_document');
+      }
+      if (aFlag & STATE_STOP || aFlag & STATE_REDIRECTING) {
+        this.requestStop(aRequest, 'progress_document');
+      }
+    } else {
+      if (aFlag & STATE_START) {
+        this.requestStart(aRequest, 'progress');
+      }
+      if (aFlag & STATE_STOP) {
+        this.requestStop(aRequest, 'progress');
+      }
+    } 
+    for (let key in this.started) {
+      var started = this.started[key];
+      this.task.debug (started.length + " " + key + " requests REMAIN:");
+      for (var i=0; i<started.length; i++) {
+        this.task.debug("   " + i + ". " + started[i].name, this.step);
+      }
     }
-    // Might be && aFlag & STATE_IS_DOCUMENT but apparently there
-    // are things that aren't documents that aren't considered to be 
-    // contained within the document (documents aren't stopped until
-    // their children are and a redirect will change the source request
-    // to STATE_STOP only once it itself has stopped so theoretically...).
-    //
-    // No ill effects so far this way beyond a lot more stuff being tracked
-    // which probably is why we had it this way as the previous heuristicky
-    // business got hung up on things now and then.
-    if (aFlag & STATE_START && (aRequest.name.slice(0, 4) == "http")) {
-      this.requestStart(aRequest, 'progress');
-    }
-    if (aFlag & STATE_STOP && (aRequest.name.slice(0, 4) == "http")) {
-      this.requestStop(aRequest, 'progress');
-    }
-    this.task.debug (started.length + " requests AFTER:");
-    for (var i=0; i<started.length; i++) {
-      this.task.debug(i + ". " + started[i].name, this.step);
-    }
-    return 0;
-  },
-
-  onLocationChange: function(aProgress, aRequest, aURI) {
-    /*
-    // Location: header is sent before any page content that could trigger
-    // a new request and .equals doesn't find it ie. it's not giving a
-    // reference to the same request object so the best we can do is assume
-    // that whatever started most recently is what caused the redirect.
-    if ((aRequest.name.slice(0, 4) == "http")) {
-      this.task.info("  Location changed to: " + aRequest.name + "(via onLocationChange)", this.step);
-      //this.task.info("  Assuming we've redirected from: " + this.started.pop().name, this.step);
-      //this.requestStart(aRequest);
-    }
-    */
     return 0;
   },
 
@@ -275,6 +272,7 @@ var progressListen = {
     this.task.debug("  Progress: " + dumpRequest.name + " cur=" + aCurSelfProgress + " max=" + aMaxSelfProgress + " curTotal=" + aCurTotalProgress + " maxTotal=" + aMaxTotalProgress, this.step);
     return 0;
   },
+  onLocationChange: function(aProgress, aRequest, aURI) {return 0;},
   onStatusChange: function() {return 0;},
   onSecurityChange: function() {return 0;},
   onLinkIconAvailable: function() {return 0;}
