@@ -1,5 +1,4 @@
 var EXPORTED_SYMBOLS = ["windowEvent"];
-Components.utils.import("resource://indexdata/runtime/core.js");
 Components.utils.import('resource://indexdata/util/logging.js');
 
 var logger = logging.getLogger("waitForLoad");
@@ -49,15 +48,15 @@ var windowEvent = {
     var context = this;
     // defined inline for closures,
     // stored with object for later removal
-    this.onEvent =  function(e) {
+    this.onEvent =  function() {
       logger.info(context.event + " dispatched to main window, page is ready");
       Components.utils.import('resource://indexdata/util/waitForLoad.js');
       waitForLoad.detatch();
     }
-    core.getBrowser().addEventListener(this.event, this.onEvent, false);
+    task.connector.getPageWindow().addEventListener(this.event, this.onEvent, false);
   },
 
   detatch: function () {
-    core.getBrowser().removeEventListener(this.event, this.onEvent, false);
+    this.task.connector.getPageWindow().removeEventListener(this.event, this.onEvent, false);
   },
 };
