@@ -1,5 +1,4 @@
 var EXPORTED_SYMBOLS = ["waitForLoad"];
-Components.utils.import("resource://indexdata/runtime/core.js");
 Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/loadDetectors/progressListen.js');
 Components.utils.import('resource://indexdata/util/loadDetectors/justObserve.js');
@@ -16,6 +15,7 @@ var waitForLoad = {
     "windowEvent":windowEvent
   },
   task: null,
+  isEngine: false,
   defaultConf: {
     detector: "progressListen",
     timeout: 30000,
@@ -39,25 +39,23 @@ var waitForLoad = {
     this.step = step;
     this.conf = conf;
     this.detector = this.detectors[conf.detector];
+    this.detector.wait(task, conf);
 
     // Interrupt the task.
     var context = this;
     this.timer = task.interrupt(conf.timeout, function () {
       if (context.detector.onTimeout) context.detector.onTimeout();
       let errorText = "Page timed out after " + conf.timeout + "ms";
-      // stop loading, it's too late now
-      core.getBrowser().stop();
       if (context.conf.errorOnTimeout) {
         context.detatch(errorText);
       } else {
-        context.task.warn("Page timed out after " + conf.timeout + "ms, continuing with DOM as-is.", context.step);
-        context.detatch();
+         context.task.warn("Page timed out after " + conf.timeout + "ms, continuing with DOM as-is.", context.step);
+         context.detatch();
       }
       return true;
     });
 
     this.startTime = +new Date();
-    this.detector.wait(task, conf);
   },
 
   detatch: function(error) {
@@ -145,7 +143,7 @@ var waitForLoad = {
     var detectorMenu = doc.getElementById("detectorMenu");
     var detectorPopup = detectorMenu.firstChild;
     var detectorSurface = doc.getElementById("specificBox");
-    for (let key in waitForLoad.detectors) {
+    for (key in waitForLoad.detectors) {
       var d = waitForLoad.detectors[key];
       var selected = false;
       if (conf.detector) {
