@@ -732,162 +732,176 @@ var xmlHelper = {
     // baseDir is often a partial file: URL, but there is no reason it
     // couldn't be an http: or other URL.
 
+    // Note that the xincludes is a 'live' nodelist, it reflects changes
+    // in the document. That means that we can always operate on its
+    // first element, and delete that one as soon as we have processed it.
+    // This allows our included documents to contain other XInclude
+    // statements. (CP-3785)
+    // This routine seems overly complex for our needs, we are not likely
+    // to do stuff with XPointers, languages, or even encodings. But let that be.
+
     resolveXIncludes: function (docu, baseDir) {
-        // http://www.w3.org/TR/xinclude/#xml-included-items
-        var xincludes = docu.getElementsByTagNameNS('http://www.w3.org/2001/XInclude', 'include');
-        if (xincludes) {
-            logger.log(Level.DEBUG, "found " + xincludes.length + " XInclude nodes");
+      // http://www.w3.org/TR/xinclude/#xml-included-items
+      var xincludes = docu.getElementsByTagNameNS('http://www.w3.org/2001/XInclude', 'include');
+      if (xincludes) {
+        logger.debug("found " + xincludes.length + " XInclude nodes to begin with");
+      }
+      let includenumber = 0; // just for debug logging
+      while (xincludes && xincludes.length > 0) {
+        includenumber++;  // just for logging
+        //for ( let xn = 0; xn < xincludes.length; xn++ )
+        //  logger.debug("xincludes list [" + xn + "]=" + xincludes[xn].getAttribute('href') );
+        var xinclude = xincludes[0]; // always process the first one
+        var href = xinclude.getAttribute('href');
+        logger.debug("handling XInclude node #" + includenumber +
+            " of " + xincludes.length + " " + href);
 
-            for (let i = 0; i < xincludes.length; i++) {
-                logger.log(Level.DEBUG, "handling XInclude node #" + (i+1) + " of " + xincludes.length);
-                var xinclude = xincludes[i];
-                var href = xinclude.getAttribute('href');
-                if (!href.match(/^[a-z]+:/)) {
-                    // It's not an absolute HREF
-                    var old = href;
-                    href = baseDir + "/" + href;
-                    //logger.log(Level.INFO, "upgraded relative href '" + old + "' to " + href);
-                }
-                var parse = xinclude.getAttribute('parse');
-                var xpointer = xinclude.getAttribute('xpointer');
-                var encoding = xinclude.getAttribute('encoding'); // e.g., UTF-8 // "text/xml or application/xml or matches text/*+xml or application/*+xml" before encoding (then UTF-8)
-                var accept = xinclude.getAttribute('accept'); // header "Accept: "+x
-                var acceptLanguage = xinclude.getAttribute('accept-language'); // "Accept-Language: "+x
-                var xiFallback = xinclude.getElementsByTagNameNS('http://www.w3.org/2001/XInclude', 'fallback')[0]; // Only one such child is allowed
-                if (href === '' || href === null) { // Points to same document if empty (null is equivalent to empty string)
-                    href = null; // Set for uniformity in testing below
-                    if (parse === 'xml' && xpointer === null) {
-                        alert('There must be an XPointer attribute present if "href" is empty an parse is "xml"');
-                        return false;
-                    }
-                }
-                else if (href.match(/#$/, '') || href.match(/^#/, '')) {
-                    alert('Fragment identifiers are disallowed in an XInclude "href" attribute');
-                    return false;
-                }
-                var j;
-                var xincludeParent = xinclude.parentNode;
-                try {
-                //netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect UniversalBrowserRead'); // Necessary with file:///-located files trying to reach external sites
-                    if (href !== null) {
-                        var response, responseType;
-                        var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                          .createInstance(Ci.nsIXMLHttpRequest);
-                        request.open('GET', href, false);
-                        request.setRequestHeader('If-Modified-Since', 'Thu, 1 Jan 1970 00:00:00 GMT');
-                        request.setRequestHeader('Cache-Control', 'no-cache');
-                        if (accept) {
-                            request.setRequestHeader('Accept', accept);
-                        }
-                        if (acceptLanguage) {
-                            request.setRequestHeader('Accept-Language', acceptLanguage);
-                        }
-                        switch (parse) {
-                            case 'text':
-                                // Priority should be on media type:
-
-                                var contentType = request.getResponseHeader('Content-Type');
-
-                                //text/xml; charset="utf-8" // Send to get headers first?
-                                // Fix: We test for file extensions as well in case file:// doesn't return content type (as seems to be the case); can some other tool be uesd in FF (or IE) to detect encoding of local file? Probably just need BOM test since other encodings must be specified
-                                var patternXML = /\.(svg|xml|xul|rdf|xhtml)$/;
-                                if ((contentType && contentType.match(/[text|application]\/(.*)\+?xml/)) || (href.indexOf('file://') === 0 && href.match(patternXML))) {
-                                    /* Grab the response as text (see below for that routine) and then find encoding within*/
-                                   var encName = '([A-Za-z][A-Za-z0-9._-]*)';
-                                   var pattern = new RegExp('^<\\?xml\\s+.*encoding\\s*=\\s*([\'"])'+encName+'\\1.*\\?>'); // Check document if not?
-                                   // Let the request be processed below
-                                }
-                                else {
-                                    if (encoding === '' || encoding === null) { // Encoding has no effect on XML
-                                        encoding = 'utf-8';
-                                    }
-                                    request.overrideMimeType('text/plain; charset='+encoding); //'x-user-defined'
-                                }
-                                responseType = 'responseText';
-                                break;
-                            case null:
-                            case 'xml':
-                                responseType = 'responseXML';
-                                break;
-                            default:
-                                alert('XInclude element contains an invalid "parse" attribute value');
-                                return false;
-                                break;
-                        }
-                        request.send(null);
-                        if((request.status === 200 || request.status === 0) && request[responseType] !== null) {
-                            response = request[responseType];
-                             if (responseType === 'responseXML') {
-                                // apply xpointer (only xpath1() subset is supported)
-                                var responseNodes;
-                                if (xpointer) {
-                                    // XPathResult.ORDERED_NODE_SNAPSHOT_TYPE was
-                                    // undefined so we use a literal 7
-                                    var xpathResult = response.evaluate(
-                                                                     xpointer,
-                                                                     response,
-                                                                     null,
-                                                                     7,
-                                                                     null
-                                                                  );
-                                    var a = [];
-                                    for(var k = 0; k < xpathResult.snapshotLength; k++) {
-                                    a[k] = xpathResult.snapshotItem(k);
-                                    }
-                                    responseNodes = a;
-                                }
-                                else { // Otherwise, the response must be a single well-formed document response
-                                    responseNodes = [response.documentElement]; // Put in array so can be treated the same way as the above
-                                }
-                                // PREPEND ANY NODE(S) (AS XML) -- REMOVE XINCLUDE LATER
-                                for (let j=0; j < responseNodes.length ; j++) {
-                                    xincludeParent.insertBefore(responseNodes[j], xinclude);
-                                }
-                             }
-                             else if (responseType === 'responseText') {
-                                 if (encName) {
-                                      var encodingType = response.match(pattern);
-                                      if (encodingType) {
-                                          encodingType = encodingType[2];
-                                      }
-                                      else {
-                                          encodingType = 'utf-8';
-                                      }
-                                      // Need to make a whole new request apparently since cannot convert the encoding after receiving it (to know what the encoding was)
-                                      var request2 = new XMLHttpRequest();
-                                      request2.overrideMimeType('text/plain; charset='+encodingType);
-                                      request2.open('GET', href, false);
-                                      request2.setRequestHeader('If-Modified-Since', 'Thu, 1 Jan 1970 00:00:00 GMT');
-                                      request2.setRequestHeader('Cache-Control', 'no-cache');
-                                      request2.send(null);
-                                      response = request2[responseType]; // Update the response for processing
-                                 }
-
-                                 // REPLACE XINCLUDE WITH THE RESPONSE AS TEXT
-                                 var textNode = docu.createTextNode(response);                             xincludeParent.replaceChild(textNode, xinclude);
-                             }
-
-                            // replace xinclude in doc with response now (as plain text or XML)
-                        }
-                    }
-                }
-                catch (e) { // Use xi:fallback if XInclude retrieval above failed
-                    if (!xiFallback) throw new Error("XInclude can't include '" + href + "': " + e);
-                    var xiFallbackChildren = xiFallback.childNodes;
-                    // PREPEND ANY NODE(S) -- REMOVE XINCLUDE LATER
-                    for (let j = 0; j < xiFallbackChildren.length ; j++) {
-                        xincludeParent.insertBefore(xiFallbackChildren[j], xinclude);
-                    }
-                }
-                logger.log(Level.DEBUG, "finished XInclude node #" + (i+1) + " of " + xincludes.length);
-            }
-            // Delete in reverse order so shrinking .length does not prematurely end the loop
-            for (let i = xincludes.length-1; i >= 0; i--) {
-                logger.log(Level.DEBUG, "deleting XInclude node #" + (i+1) + " of " + xincludes.length);
-                xincludes[i].parentNode.removeChild(xincludes[i]);
-            }
+        if (!href.match(/^[a-z]+:/)) {
+          // It's not an absolute HREF
+          var old = href;
+          href = baseDir + "/" + href;
+          //logger.log(Level.INFO, "upgraded relative href '" + old + "' to " + href);
         }
-        return docu;
-    }
+        var parse = xinclude.getAttribute('parse');
+        var xpointer = xinclude.getAttribute('xpointer');
+        var encoding = xinclude.getAttribute('encoding'); // e.g., UTF-8 // "text/xml or application/xml or matches text/*+xml or application/*+xml" before encoding (then UTF-8)
+        var accept = xinclude.getAttribute('accept'); // header "Accept: "+x
+        var acceptLanguage = xinclude.getAttribute('accept-language'); // "Accept-Language: "+x
+        var xiFallback = xinclude.getElementsByTagNameNS('http://www.w3.org/2001/XInclude', 'fallback')[0]; // Only one such child is allowed
+        if (href === '' || href === null) { // Points to same document if empty (null is equivalent to empty string)
+          href = null; // Set for uniformity in testing below
+          if (parse === 'xml' && xpointer === null) {
+            alert('There must be an XPointer attribute present if "href" is empty an parse is "xml"');
+            return false;
+          }
+        }
+        else if (href.match(/#$/, '') || href.match(/^#/, '')) {
+          alert('Fragment identifiers are disallowed in an XInclude "href" attribute');
+          return false;
+        }
+        var xincludeParent = xinclude.parentNode;
+        try {
+        //netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect UniversalBrowserRead'); // Necessary with file:///-located files trying to reach external sites
+          if (href !== null) {
+            var response, responseType;
+            var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
+              .createInstance(Ci.nsIXMLHttpRequest);
+            request.open('GET', href, false);
+            request.setRequestHeader('If-Modified-Since', 'Thu, 1 Jan 1970 00:00:00 GMT');
+            request.setRequestHeader('Cache-Control', 'no-cache');
+            if (accept) {
+              request.setRequestHeader('Accept', accept);
+            }
+            if (acceptLanguage) {
+              request.setRequestHeader('Accept-Language', acceptLanguage);
+            }
+            switch (parse) {
+              case 'text':
+                // Priority should be on media type:
+                var contentType = request.getResponseHeader('Content-Type');
+
+                //text/xml; charset="utf-8" // Send to get headers first?
+                // Fix: We test for file extensions as well in case 'file://'
+                // doesn't return content type (as seems to be the case);
+                // can some other tool be uesd in FF (or IE) to detect encoding
+                // of local file? Probably just need BOM test since other
+                // encodings must be specified
+                var patternXML = /\.(svg|xml|xul|rdf|xhtml)$/;
+                if ((contentType && contentType.match(/[text|application]\/(.*)\+?xml/)) || (href.indexOf('file://') === 0 && href.match(patternXML))) {
+                  // Grab the response as text (see below for that routine) and then find encoding within
+                  var encName = '([A-Za-z][A-Za-z0-9._-]*)';
+                  var pattern = new RegExp('^<\\?xml\\s+.*encoding\\s*=\\s*([\'"])'+encName+'\\1.*\\?>');
+                  // Check document if not?
+                  // Let the request be processed below
+                }
+                else {
+                  if (encoding === '' || encoding === null) { // Encoding has no effect on XML
+                    encoding = 'utf-8';
+                  }
+                  request.overrideMimeType('text/plain; charset='+encoding); //'x-user-defined'
+                }
+                responseType = 'responseText';
+                break;
+              case null:
+              case 'xml':
+                responseType = 'responseXML';
+                break;
+              default:
+                alert('XInclude element contains an invalid "parse" attribute value');
+                return false;
+                break;
+            }
+            request.send(null);
+            if ( (request.status === 200 || request.status === 0) &&
+                  request[responseType] !== null) {
+              response = request[responseType];
+              if (responseType === 'responseXML') {
+                // apply xpointer (only xpath1() subset is supported)
+                var responseNodes;
+                if (xpointer) {
+                  // XPathResult.ORDERED_NODE_SNAPSHOT_TYPE was
+                  // undefined so we use a literal 7
+                  var xpathResult = response.evaluate( xpointer, response,
+                                 null, 7, null );
+                  var a = [];
+                  for(var k = 0; k < xpathResult.snapshotLength; k++) {
+                    a[k] = xpathResult.snapshotItem(k);
+                  }
+                  responseNodes = a;
+                }
+                else { // Otherwise, the response must be a single well-formed document response
+                  responseNodes = [response.documentElement]; // Put in array
+                               // so can be treated the same way as the above
+                }
+                // PREPEND ANY NODE(S) (AS XML) -- REMOVE XINCLUDE LATER
+                for (let j=0; j < responseNodes.length ; j++) {
+                  xincludeParent.insertBefore(responseNodes[j], xinclude);
+                }
+              }
+              else if (responseType === 'responseText') {
+                if (encName) {
+                  var encodingType = response.match(pattern);
+                  if (encodingType) {
+                    encodingType = encodingType[2];
+                  }
+                  else {
+                    encodingType = 'utf-8';
+                  }
+                  // Need to make a whole new request apparently since cannot
+                  // convert the encoding after receiving it (to know what the
+                  // encoding was)
+                  var request2 = new XMLHttpRequest();
+                  request2.overrideMimeType('text/plain; charset='+encodingType);
+                  request2.open('GET', href, false);
+                  request2.setRequestHeader('If-Modified-Since', 'Thu, 1 Jan 1970 00:00:00 GMT');
+                  request2.setRequestHeader('Cache-Control', 'no-cache');
+                  request2.send(null);
+                  response = request2[responseType]; // Update the response for processing
+                }
+
+                // REPLACE XINCLUDE WITH THE RESPONSE AS TEXT
+                var textNode = docu.createTextNode(response);
+                xincludeParent.replaceChild(textNode, xinclude);
+              }
+            }
+          }
+        }
+        catch (e) { // Use xi:fallback if XInclude retrieval above failed
+          logger.debug("Caught an error " + e + " when processing XIncludes. fb=" + xiFallback );
+          if (!xiFallback)
+            throw new Error("XInclude can't include '" + href + "': " + e);
+          var xiFallbackChildren = xiFallback.childNodes;
+          // PREPEND ANY NODE(S) -- REMOVE XINCLUDE LATER
+          for (let j = 0; j < xiFallbackChildren.length ; j++) {
+            xincludeParent.insertBefore(xiFallbackChildren[j], xinclude);
+          }
+        }
+        // Delete the node we just processed. The xincludes is a live list, so it's [0] will
+        // then be the next xinclude to handle - even if that came from the included stuff.
+        logger.log(Level.DEBUG, "deleting XInclude node #" + includenumber + " of " + xincludes.length);
+        xinclude.parentNode.removeChild(xinclude);
+        } // while
+      return docu;
+    } // resolveXIncludes
 
 } // xmlHelper
