@@ -432,11 +432,11 @@ XpatternTextParser.prototype._end = function() {
 var oktests = [
   "TD",
   "TD $variable [ @attr=\"value\" ] ",
-  "TD $variable [ @attr ~ \"value\" ] ",
+  "TD $variable [ @attr ~ /value/ ] ",
   "A [ @href = \"author\" ]",
-  "A [ @href ~ \"author\" ]",
-  "tag-with-dashes [ @href ~ \"author\" ]",
-  //"TD $variable [ @attr ~ /value/ ] ", // TODO not implemented yet. Dumps wrong
+  "A [ @href ~ /author/ ]",
+  "tag-with-dashes [ @href ~ /author/ ]",
+  "A [ @href ~ \"foo/bar\" ]",
   "TD $var -html",
   "TD $var -html -xml",
   "TD $var [ @attr $attrvar ]",
