@@ -6,14 +6,11 @@ echo "running regression in $DIR"
 GRET=0
 : ${diff_opt="-u"}
 : ${REGRESSION_FATAL=false}
+: ${MAKE_OUT=false}
 
-GRET=0
 error () {
     $REGRESSION_FATAL && exit 1
-
-    plus=$1
-    test -n "$plus" || plus=1   
-    GRET=`expr $GRET + $plus`
+    GRET=1
 }
 
 # TODO - Thetest-ws script assumes a keyword search only, so
@@ -34,17 +31,21 @@ for cfile in $CF_CONNECTORS/*.cf $CF_CONNECTORS/$PLAINDIR/*.cf; do
 	diff $diff_opt $DIR/$c.out $DIR/$c.log
 	if test $? -ne 0; then
 	    echo "$c: differ"
-            error 1
+            error
 	else
 	    echo "  $c: OK"
 	fi
+    elif $MAKE_OUT; then
+        echo "$c Making first time"
+        cp $DIR/$c.log $DIR/$c.out
+        error 
     else
         echo "$c FAIL!"
         echo "  Missing file: $c.out"
         echo "  Please check that $c.log looks right, and copy it to $c.out"
         echo "  And add it to git with "
         echo "    cd $DIR; git add $c.out"
-        error 1
+        error
     fi
 done
 exit $GRET
