@@ -44,7 +44,6 @@ ConnectorRuntime.prototype = {
     resProt.setSubstitution("indexdata", aliasURI);
     //set up core metainfo
     Components.utils.import('resource://indexdata/runtime/core.js');
-    this.core = core;
     core.set('inBuilder', false);
     let info = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
     core.set('version', info.version);
@@ -65,22 +64,10 @@ ConnectorRuntime.prototype = {
   observe: function(subject, topic, data) {
   },
   setBrowser : function (browser) {
-    var win = browser.contentDOMWindow;
-    var doc = win.document;
-    // we can't include the browser in engine.xul because the type attribute
-    // needs to be set before it's added to the document
-    var browser =  doc.createElement("browser");
-    //browser.setAttribute("type", "content-primary");
-    browser.setAttribute("flex", "1");
-    browser.setAttribute("autocompleteenabled", false);
-    browser.setAttribute("autoscroll", false);
-    // TODO: add history to our embedding for use with harvesting, etc.
-    browser.setAttribute("disablehistory", true);
-    // doesn't disable actual security, just nsISecureBrowserUI
-    browser.setAttribute("disablesecurity", true);
-    doc.getElementById("main-window").appendChild(browser);
-    //var browser = doc.getElementById("content");
-    this.core.setBrowser(browser);
+    this.connector.setBrowser(browser);
+  },
+  setBrowserWindow : function (browserWindow) {
+    this.connector.setPageWindow(browserWindow);
   },
   loadConf: function (filePath, sessionJSON, errormessage) {
     try {
