@@ -126,6 +126,46 @@ static int CheckDisplay(const char *cstr)
     return 0;
 }
 
+bool CF_Display::terminate_pid(long pid)
+{
+    int i;
+    for (i = 0; i < 10; i++)
+    {
+        int r = kill(pid, i < 5 ? SIGTERM : SIGKILL);
+        int x = errno;
+        if (m_logger)
+        {
+            if (r == -1)
+                m_logger->printf("engine", YLOG_WARN,
+                                 "display: kill Xvfb PID=%ld: %s",
+                                 (long) pid, strerror(x));
+            else
+                m_logger->printf("engine", YLOG_WARN,
+                                 "display: kill Xvfb PID=%ld",
+                                 (long) pid);
+        }
+        if (r == -1)
+        {
+            if (x == ESRCH)
+                return true; /* process is gone */
+            else
+            {
+                wrbuf_printf(wrbuf_error, "display: unable to kill Xvfb "
+                             "PID=%ld: %s", (long) pid, strerror(x));
+                return false;
+            }
+        }
+        waitpid(pid, 0, WNOHANG);
+        struct timespec req;
+        req.tv_sec = 0;
+        req.tv_nsec = 50000000; /* 1/20 sec */
+        ::nanosleep(&req, 0);
+    }
+    wrbuf_printf(wrbuf_error, "display: unable to kill Xvfb PID=%ld after "
+                 "10 tries", (long) pid);
+    return false;
+}
+
 bool CF_Display::lock_via_xvfb_cmd(const char *lockdir, const char *xvfb_cmd,
                                    int display_base, int log_fd)
 {
@@ -236,40 +276,10 @@ bool CF_Display::lock_via_xvfb_cmd(const char *lockdir, const char *xvfb_cmd,
 
     wrbuf_destroy(x11_lock_fname);
 
-    if (pid > 0)
+    if (pid > 0 && !terminate_pid(pid))
     {
-        while (1)
-        {
-            int r = kill(pid, SIGTERM);
-            int x = errno;
-            if (m_logger)
-            {
-                if (r == -1)
-                    m_logger->printf("engine", YLOG_WARN,
-                                     "display: kill stale Xvfb PID=%ld: %s",
-                                     (long) pid, strerror(x));
-                else
-                    m_logger->printf("engine", YLOG_WARN,
-                                     "display: kill stale Xvfb PID=%ld",
-                                     (long) pid);
-            }
-            if (r == -1)
-            {
-                if (x == ESRCH)
-                    break; /* stale process is gone */
-                else
-                {
-                    wrbuf_printf(wrbuf_error, "display: unable to kill stale Xvfb "
-                                 "PID=%ld: %s", (long) pid, strerror(x));
-                    close();
-                    return false;
-                }
-            }
-            struct timespec req;
-            req.tv_sec = 0;
-            req.tv_nsec = 50000000; /* 1/20 sec */
-            ::nanosleep(&req, 0);
-        }
+        close();
+        return false;
     }
     wrbuf_printf(fname_log_file, "%s/Xvfb.%d.log", lockdir, display_base + dno);
 
@@ -405,38 +415,18 @@ bool CF_Display::lock_via_xvfb_cmd(const char *lockdir, const char *xvfb_cmd,
     wrbuf_rewind(wrbuf_error);
     wrbuf_printf(wrbuf_error, "Xvfb start failure. DISPLAY=%s",
                  wrbuf_cstr(m_display));
+    xvfb_pid = 0;
     close();
     return false;
 }
 
 void CF_Display::close()
 {
-    if (xvfb_pid)
-    {
-        int r = kill(xvfb_pid, SIGTERM);
-        int x = errno;
-
-        if (m_logger)
-        {
-            if (r == 0)
-                m_logger->printf("engine", YLOG_DEBUG,
-                                 "display: kill Xvfb DISPLAY=%s PID=%ld",
-                                 wrbuf_cstr(m_display), (long) xvfb_pid);
-            else
-                m_logger->printf("engine", YLOG_WARN,
-                                 "display: kill Xvfb DISPLAY=%s PID=%ld: %s",
-                                 wrbuf_cstr(m_display), (long) xvfb_pid,
-                                 strerror(x));
-        }
-        if (r == 0)
-        {
-            int status;
-            waitpid(xvfb_pid, &status, 0);
-        }
-    }
     if (unlink_fname && wrbuf_len(fname_lck_file))
         ::unlink(wrbuf_cstr(fname_lck_file));
 
+    if (xvfb_pid)
+        terminate_pid(xvfb_pid);
     xvfb_pid = 0;
     if (cf_lck_fd != -1)
     {
