@@ -26,6 +26,7 @@ CF_Factory::~CF_Factory()
     wrbuf_destroy(cf_connector_path);
     wrbuf_destroy(cf_repo_fetch_url);
     wrbuf_destroy(cf_repo_auth_url);
+    wrbuf_destroy(cfmain_path);
     wrbuf_destroy(tmp_dir);
 }
 
@@ -36,6 +37,7 @@ CF_Factory::CF_Factory(const xmlNode *n)
     const char *connector_path = 0;
     const char *app_path = 0;
     const char *profile_path = 0;
+    const char *cfmain_path = 0;
     const char *repo_fetch_url = 0;
     const char *repo_auth_url = 0;
     const char *repo_proxy = 0;
@@ -71,6 +73,8 @@ CF_Factory::CF_Factory(const xmlNode *n)
                     app_path = (const char *) attr->children->content;
                 else if (!strcmp((const char *) attr->name, "profile_path"))
                     profile_path = (const char *) attr->children->content;
+                else if (!strcmp((const char *) attr->name, "cfmain_path"))
+                    cfmain_path = (const char *) attr->children->content;
                 else if (!strcmp((const char *) attr->name, "repo_fetch_url"))
                     repo_fetch_url = (const char *) attr->children->content;
                 else if (!strcmp((const char *) attr->name, "repo_auth_url"))
@@ -182,6 +186,15 @@ CF_Factory::CF_Factory(const xmlNode *n)
     }
     else
         this->cf_profile_path = 0;
+
+    if (cfmain_path)
+    {
+        this->cfmain_path = wrbuf_alloc();
+        wrbuf_puts(this->cfmain_path, cfmain_path);
+    }
+    else
+        this->cfmain_path = 0;
+
     wrbuf_destroy(base_path_wr);
 }
 
@@ -348,17 +361,18 @@ WRBUF CF_Factory::get_cf_file(WRBUF username, WRBUF password, const char *db, in
 
 CF_Engine *CF_Factory::create(enum CF_Engine::Flags flags, const char *proxy,
                               const char *loglevel,
-                              CF_Logger *logger, int fd, int capt_stdout)
+                              const char *sesname, int capt_fd)
 {
     return new CF_Engine(
         flags,
         wrbuf_cstr(cf_module_path),
         (cf_app_path ? wrbuf_cstr(cf_app_path) : 0 ),
         (cf_profile_path ? wrbuf_cstr(cf_profile_path) : 0 ),
+        (cfmain_path ? wrbuf_cstr(cfmain_path) : 0),
         proxy,
         loglevel,
-        logger,
-        fd, capt_stdout);
+        sesname,
+        capt_fd);
 }
 
 void CF_Factory::auth(WRBUF username, WRBUF password, int *http_status)
