@@ -39,7 +39,7 @@ public:
      */
     CF_Engine *create(enum CF_Engine::Flags, const char *proxy,
                       const char *loglevel,
-                      CF_Logger *logger, int fd, int capt_stdout);
+                      const char *sesname, int capt_fd);
 
     WRBUF get_cf_file(WRBUF username, WRBUF password,
                       const char *db, int *http_status);
@@ -58,6 +58,7 @@ private:
     WRBUF cf_repo_fetch_url;
     WRBUF cf_repo_auth_url;
     WRBUF cf_repo_proxy;
+    WRBUF cfmain_path;
     WRBUF tmp_dir;
 };
 
