@@ -15,8 +15,8 @@ int main(int argc, char **argv)
     }
     try
     {
-        CF_Engine e(CF_Engine::FORK, getenv("CF_MODULE_PATH"), 0, 0, 0, 0,
-                    0 /* TODO: LOGGER */, -1, 1);
+        CF_Engine e(CF_Engine::FORK, getenv("CF_MODULE_PATH"), 0, 0, 0, 0, 0,
+                    0 /* TODO: LOGGER */, -1);
 	e.load_cf(argv[1]);
 
 	const char *result = 0;
