@@ -124,6 +124,11 @@ public:
     nsresult CreateBrowser(void* aNativeWindow, PRInt32 x, PRInt32 y,
 			   PRInt32 width, PRInt32 height, PRUint32 chromeFlags = 0);
 
+    /**
+     * Return (void) pointer to browser object
+     */
+    void* GetBrowser();
+
     /**
      * Sets location and dimension of the browser window.
      * Call this when resizing.
