@@ -13,6 +13,11 @@
 #include <yaz/xmalloc.h>
 #include <yaz/options.h>
 
+static void usage(void)
+{
+    printf("json_parse [-x][-f]\n");
+    exit(1);
+}
 #include "json.h"
 /** \brief main as usual */
 int main(int argc, char **argv)
@@ -20,16 +25,21 @@ int main(int argc, char **argv)
     char *arg = 0;
     int r;
     bool xml_parse = false;
-    while ((r=options("x", argv, argc, &arg)) != YAZ_OPTIONS_EOF)
+    bool facet_parse = false;
+    while ((r=options("xf", argv, argc, &arg)) != YAZ_OPTIONS_EOF)
     {
         switch(r)
         {
         case YAZ_OPTIONS_ERROR:
             fprintf(stderr, "Bad option: %s\n", arg);
+            usage();
             exit(1);
         case 'x':
             xml_parse = true;
             break;
+        case 'f':
+            facet_parse = true;
+            break;
         }
     }
     WRBUF w = wrbuf_alloc();
@@ -48,12 +58,33 @@ int main(int argc, char **argv)
         exit(1);
     }
     wrbuf_rewind(w);
-    if (xml_parse)
+    if (facet_parse)
+    {
+        ODR odr = odr_createmem(ODR_ENCODE);
+        ODR print  = odr_createmem(ODR_PRINT);
+        Z_FacetList *facet_list = json_to_facet(n, odr);
+        if (facet_list)
+        {
+            z_FacetList(print, &facet_list, 0, 0);
+            wrbuf_puts(w, "{\"facets\":");
+            facet_to_json(facet_list, w);
+            wrbuf_puts(w, "}");
+            puts(wrbuf_cstr(w));
+        }
+        odr_destroy(odr);
+        odr_destroy(print);
+    }
+    else if (xml_parse)
+    {
         json_to_xml(n, w, "opt");
+        puts(wrbuf_cstr(w));
+    }
     else
+    {
         json_write_wrbuf(n, w);
+        puts(wrbuf_cstr(w));
+    }
     json_remove_node(n);
-    puts(wrbuf_cstr(w));
     wrbuf_destroy(w);
     exit(0);
 }
