@@ -32,6 +32,7 @@
 #include <yaz/snprintf.h>
 #include <sys/uio.h>
 #include <fcntl.h>
+#include <signal.h>
 
 #include "cf_assert.h"
 #include "cf_factory.h"
@@ -63,10 +64,11 @@ namespace metaproxy_1 {
             void configure(const xmlNode * ptr, bool test_only,
                            const char *path);
             void start() const;
+            void stop(int signo) const;
         private:
             boost::scoped_ptr<Rep> m_p;
         };
-        class CF::WSession : CF_Logger {
+        class CF::WSession {
             friend class CF;
             friend class Rep;
             CF_Engine *engine;
@@ -85,7 +87,6 @@ namespace metaproxy_1 {
             mp::wrbuf log_wr;
         public:
             ~WSession();
-            void printf(const char *module, int level, const char *fmt, ...);
         };
 
         class CF::ZSession {
@@ -177,6 +178,7 @@ namespace metaproxy_1 {
 
             CF_Factory *cf_factory;
 
+            int stop_signo;
             int max_age;
             int m_connector_next_id;
 
@@ -254,6 +256,7 @@ yf::CF::ZSession::ZSession(const Z_IdAuthentication *auth)
 
 yf::CF::ZSession::~ZSession()
 {
+    cf_session.reset();
     close();
     wrbuf_destroy(username);
     wrbuf_destroy(password);
@@ -463,32 +466,6 @@ bool yf::CF::WSession::module_check(const char *actual)
     return false;
 }
 
-void yf::CF::WSession::printf(const char *module, int level,
-                              const char *fmt, ...)
-{
-    va_list ap;
-    char buf[4096];
-
-    va_start(ap, fmt);
-    yaz_vsnprintf(buf, sizeof(buf)-1, fmt, ap);
-
-    const char *lead = "[JavaScript Warning: ";
-    if (strncmp(lead, buf, strlen(lead)))
-    {
-        yaz_log(level, "%s %s %s", session_name.c_str(), module, buf);
-        if (log_file != -1 && module_check(module))
-        {
-            boost::mutex::scoped_lock lock(log_mutex);
-
-            wrbuf_rewind(log_wr);
-            wrbuf_printf(log_wr, "%s %s %s\n", CF_Logger::logstr(level),
-                         module, buf);
-            write(log_file, log_wr.buf(), log_wr.len());
-        }
-    }
-    va_end(ap);
-}
-
 void yf::CF::WSession::log(int level, const char *fmt, ...)
 {
     const char *module = "engine";
@@ -534,6 +511,7 @@ yf::CF::Rep::Rep()
     cf_factory = 0;
     http_prefix = "connector";
     max_age = 5;
+    stop_signo = 0;
     enable_z3950 = false;
     m_connector_next_id = 1;
     close_down = false;
@@ -552,7 +530,7 @@ yf::CF::Rep::~Rep()
 
 yf::CF::CF() : m_p(new CF::Rep)
 {
-
+    yaz_log(YLOG_LOG, "CF %s %s", CF_VERSION, VERSION_SHA1);
 }
 
 yf::CF::~CF()
@@ -639,10 +617,15 @@ CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
         return 0;
     }
 
+    if (stop_signo == SIGTERM)
+    {
+        ses->log(YLOG_LOG, "CF: sigterm");
+        return 0;
+    }
     CF_Engine *cf = 0;
     try {
-        cf = cf_factory->create(engine_flag, proxy, loglevel, ses, log_file,
-                                ses->module_check("stdout") ? 1 : 0);
+        cf = cf_factory->create(engine_flag, proxy, loglevel,
+                                ses->session_name.c_str(), log_file);
     }
     catch (CF_Engine_Error &e)
     {
@@ -989,6 +972,8 @@ void yf::CF::Rep::release_zsession(Package &package)
             m_cond_session_ready.notify_all();
         }
     }
+    if (r_session)
+        r_session->close();
     // r_session goes out of scope and ZSession may be destructed
 }
 
@@ -1837,6 +1822,11 @@ void yf::CF::start() const
     m_p->m_thrds.add_thread(new boost::thread(w));
 }
 
+void yf::CF::stop(int signo) const
+{
+    m_p->stop_signo = signo;
+}
+
 void yf::CF::process(Package &package) const
 {
     Z_GDU *gdu = package.request().get();
