@@ -35,7 +35,7 @@ it=0
 maxit=240
 while true; do
     rm -f mp.log
-    CF_DISPLAY_LOCK=/tmp/cf-xvfb-$(logname) CF_APP_PATH=`pwd` CF_TMP_DIR=`pwd` \
+    CF_DISPLAY_LOCK=/tmp/cf-xvfb-$(whoami) CF_APP_PATH=`pwd` CF_TMP_DIR=`pwd` \
 	$MP_PROG -c mp_config.xml -D -l mp.log -p mp.pid
     if test $? -eq 0; then
 	break
