@@ -43,11 +43,11 @@ static struct querymap qmap[] = {
     /* 8 */ { "subject", { 21, 47, 0 } }
 };
 
-const int n_qmaps = (sizeof(qmap)/sizeof(*qmap));
+static const int n_qmaps = (sizeof(qmap)/sizeof(*qmap));
 
-const int singleyearidx = 1;
-const int startyearidx = 2;
-const int endyearidx = 3;
+static const int singleyearidx = 1;
+static const int startyearidx = 2;
+static const int endyearidx = 3;
 
 struct queryvalue {
     WRBUF paramname;
@@ -517,6 +517,7 @@ void write_sortkey(wrbuf *w, struct sort_elem *sort_list)
 bool CF_RPN_to_JSON::parse(Z_RPNQuery *q, const char *sub_database,
                            struct sort_elem *sort_list)
 {
+    bool ret = false;
     int i;
     for (i = 0; i < json_results_max; i++)
         wrbuf_rewind(json_results[i]);
@@ -538,46 +539,48 @@ bool CF_RPN_to_JSON::parse(Z_RPNQuery *q, const char *sub_database,
     // TODO - Set the first names from qmaps
 
     // Extract the real search terms
-    if (!rpnstructure(q->RPNStructure, &qv))
-        return false;
-
-    // Add sort stuff into qv
-    if (sort_list)
+    if (rpnstructure(q->RPNStructure, &qv))
     {
-        int idx = findstringattridx("sortkey", &qv);
-        if (qv.v[idx].value == 0)
-            // should always be.
-            qv.v[idx].value = wrbuf_alloc();
-        else
-            wrbuf_puts(qv.v[idx].value, " ");
-        write_sortkey( qv.v[idx].value, sort_list);
+        ret = true;
+
+        // Add sort stuff into qv
+        if (sort_list)
+        {
+            int idx = findstringattridx("sortkey", &qv);
+            if (qv.v[idx].value == 0)
+                // should always be.
+                qv.v[idx].value = wrbuf_alloc();
+            else
+                wrbuf_puts(qv.v[idx].value, " ");
+            write_sortkey( qv.v[idx].value, sort_list);
+        }
+
+        /* Date magic, part 1: If we have only one year,
+         * pass it first in the single 'year' argument */
+        if (( qv.v[startyearidx].value != 0) &&
+            ( qv.v[endyearidx].value != 0) &&
+            ( strcmp( wrbuf_cstr(qv.v[startyearidx].value),
+                      wrbuf_cstr(qv.v[endyearidx].value)) == 0))
+        {
+            // full date range as second..
+            build_query(&qv, sub_database, json_results[1]);
+
+            // everything to singleyear..
+            qv.v[singleyearidx].value = qv.v[startyearidx].value; // transfer
+            qv.v[startyearidx].value = 0;
+
+            wrbuf_destroy(qv.v[endyearidx].value);  // delete
+            qv.v[endyearidx].value = 0;
+        }
+        // normal query including single date
+        build_query(&qv, sub_database, json_results[0]);
     }
-
-    /* Date magic, part 1: If we have only one year,
-     * pass it first in the single 'year' argument */
-    if (( qv.v[startyearidx].value != 0) &&
-        ( qv.v[endyearidx].value != 0) &&
-        ( strcmp( wrbuf_cstr(qv.v[startyearidx].value),
-                  wrbuf_cstr(qv.v[endyearidx].value)) == 0))
-    {
-        // full date range as second..
-        build_query(&qv, sub_database, json_results[1]);
-
-        // everything to singleyear..
-        qv.v[singleyearidx].value = qv.v[startyearidx].value; // transfer
-        qv.v[startyearidx].value = 0;
-
-        wrbuf_destroy(qv.v[endyearidx].value);  // delete
-        qv.v[endyearidx].value = 0;
-    }
-    // normal query including single date
-    build_query(&qv, sub_database, json_results[0]);
     for (i = 0; i < max_queryvalues; i++)
     {
         wrbuf_destroy(qv.v[i].value);
         wrbuf_destroy(qv.v[i].paramname);
     }
-    return true;
+    return ret;
 }
 
 const char *CF_RPN_to_JSON::get_result(int no)
@@ -610,7 +613,7 @@ int CF_RPN_to_JSON::get_diagnostic(const char **addinfo)
 
 //////////// parse_full stuff
 
-const int n_attrtypes = 6;
+static const int n_attrtypes = 6;
 
 struct attrmap_t {
     int attrvalue;
@@ -622,7 +625,7 @@ struct attrinfo_t {
     const attrmap_t values[20]; // We won't have more than that!
 };
 
-const struct attrinfo_t attributes[] = {
+static const struct attrinfo_t attributes[] = {
     {  "", { {0,0} } }, /* unused, to get base-1 array */
     {  "field",
        { {0,0} }  /* we do not support numerical use attributes */
