@@ -9,6 +9,7 @@
 
 #include <yaz/wrbuf.h>
 #include <yaz/z-core.h>
+#include <yaz/z-facet-1.h>
 
 struct queryvalues;
 
@@ -21,7 +22,7 @@ public:
                struct sort_elem *sort_list);
     bool parse_full(Z_RPNQuery *q,
                     const char *sub_database,
-                    struct sort_elem *sort_list);
+                    struct sort_elem *sort_list, Z_FacetList *facet_list);
     const char *get_result(int no);
     const char *get_full();
     int get_diagnostic(const char **addinfo);
