@@ -105,7 +105,7 @@ int main(int argc, char **argv)
             rpn_to_json.set_capability_flags(capability_flags);
 
             if ( do_full ) {
-                if (rpn_to_json.parse_full(rpn, sub_database,sort_list))
+                if (rpn_to_json.parse_full(rpn, sub_database,sort_list, 0))
                 {
                     const char *json_res;
                     if ((json_res = rpn_to_json.get_full()))
