@@ -6,7 +6,7 @@
 : ${DEBUG=1}
 log_dir=logs
 
-export CF_DISPLAY_LOCK=/tmp/cf-xvfb-$(logname)
+export CF_DISPLAY_LOCK=/tmp/cf-xvfb-$(whoami)
 case `uname` in 
    Linux ) ;;
        * ) export CF_APP_PATH=`pwd` ;;
@@ -22,6 +22,7 @@ do
   if [ $DEBUG -gt 0 ]; then
     ./cfrun -u $U || exit 1
   else
+    mkdir -p $log_dir
     out=$log_dir/`echo $U | sed 's,/,-,g'`.log
     if ./cfrun -u $U > $out 2>&1; then
     	egrep '^Unit test' $out
